
#include "common.h"
#include "Renderer.h"


using namespace sys;


extern "C"
{
JNIEXPORT int JNICALL		Java_sys_BaseActivity_initNative(JNIEnv*, jobject, jint, jint, jobject);
JNIEXPORT jboolean JNICALL	Java_sys_BaseActivity_updateNative(JNIEnv*, jobject);
}

void	init_app(void);			// メイン初期化
Bool	update_app(void);		// メイン稼働

/************
    初期化
 ************/
JNIEXPORT int JNICALL	Java_sys_BaseActivity_initNative(JNIEnv* env, jobject, jint width, jint height, jobject mgr)
{
	LOGI("initNative (%d, %d)", width, height);

	asset_manager = AAssetManager_fromJava(env, mgr);		// asset読み込みマネージャー
	assert(asset_manager != NULL);

	Renderer::init(width, height);				// 描画管理初期化

	init_app();									// アプリメイン初期化

	return	FRAME_PERIOD;
}

/**********
    稼働
 **********/
JNIEXPORT jboolean JNICALL	Java_sys_BaseActivity_updateNative(JNIEnv*, jobject)
{
	Renderer::update();							// 描画管理稼働

	if ( update_app() ) {						// アプリメイン稼働
		return	JNI_TRUE;
	}
	Renderer::quit();							// 描画管理終了
	return	JNI_FALSE;							// アプリ終了
}


namespace sys
{

AAssetManager*	asset_manager;			// asset読み込み用

/****************************************
    assetファイル読み込み
			引数	name = ファイル名
			戻り値	データ
					size：データサイズ
 ****************************************/
void*	load_asset(const char* name, u32* size)
{
	AAsset*	_asset = AAssetManager_open(asset_manager, name, AASSET_MODE_BUFFER);

	assert(_asset);

	size_t	_size = AAsset_getLength(_asset);		// ファイルサイズ
	void*	_buf = malloc(_size);					// データバッファ

	AAsset_read(_asset, _buf, _size);				// データ読み込み
	AAsset_close(_asset);
	if ( size ) {
		*size = (u32)_size;
	}
	return	_buf;
}

}

/**************** End of File *************************************************/